searchNodes=[{"ref":"BatchLoader.html","title":"BatchLoader","type":"module","doc":"A struct which is being used for batching."},{"ref":"BatchLoader.html#cache_key/1","title":"BatchLoader.cache_key/1","type":"function","doc":""},{"ref":"BatchLoader.Absinthe.html","title":"BatchLoader.Absinthe","type":"module","doc":"A module which integrates BatchLoader with Absinthe."},{"ref":"BatchLoader.Absinthe.html#for/3","title":"BatchLoader.Absinthe.for/3","type":"function","doc":"Creates a BatchLoader struct and calls the BatchLoader.Absinthe.Middleware. Example iex&gt; BatchLoader.Absinthe.for(post.user_id, batch_function)"},{"ref":"BatchLoader.Absinthe.Middleware.html","title":"BatchLoader.Absinthe.Middleware","type":"module","doc":"Absinthe Middleware which delays the resolution and then gets the result from CacheStore."},{"ref":"BatchLoader.Absinthe.Middleware.html#call/2","title":"BatchLoader.Absinthe.Middleware.call/2","type":"function","doc":"This is the main middleware callback. It receives an %Absinthe.Resolution{} struct and it needs to return an %Absinthe.Resolution{} struct. The second argument will be whatever value was passed to the middleware call that setup the middleware. Callback implementation for Absinthe.Middleware.call/2."},{"ref":"BatchLoader.Absinthe.Middleware.html#call/3","title":"BatchLoader.Absinthe.Middleware.call/3","type":"function","doc":""},{"ref":"BatchLoader.Absinthe.Plugin.html","title":"BatchLoader.Absinthe.Plugin","type":"module","doc":"Absinthe Plugin which re-runs the delayed resolution and executes batching."},{"ref":"BatchLoader.Absinthe.Plugin.html#after_resolution/1","title":"BatchLoader.Absinthe.Plugin.after_resolution/1","type":"function","doc":"callback to do something with the resolution accumulator after resolution. NOTE: This function is given the full accumulator. Namespacing is suggested to avoid conflicts. Callback implementation for Absinthe.Plugin.after_resolution/1."},{"ref":"BatchLoader.Absinthe.Plugin.html#before_resolution/1","title":"BatchLoader.Absinthe.Plugin.before_resolution/1","type":"function","doc":"callback to setup the resolution accumulator prior to resolution. NOTE: This function is given the full accumulator. Namespacing is suggested to avoid conflicts. Callback implementation for Absinthe.Plugin.before_resolution/1."},{"ref":"BatchLoader.Absinthe.Plugin.html#pipeline/2","title":"BatchLoader.Absinthe.Plugin.pipeline/2","type":"function","doc":"callback used to specify additional phases to run. Plugins may require additional resolution phases to be run. This function should use values set in the resolution accumulator to determine whether or not additional phases are required. NOTE: This function is given the whole pipeline to be inserted after the current phase completes. Callback implementation for Absinthe.Plugin.pipeline/2."},{"ref":"BatchLoader.Cache.html","title":"BatchLoader.Cache","type":"module","doc":"A struct which is used like cache per batch function."},{"ref":"BatchLoader.Cache.html#add_item/2","title":"BatchLoader.Cache.add_item/2","type":"function","doc":""},{"ref":"BatchLoader.Cache.html#batch/1","title":"BatchLoader.Cache.batch/1","type":"function","doc":""},{"ref":"BatchLoader.Cache.html#batched?/1","title":"BatchLoader.Cache.batched?/1","type":"function","doc":""},{"ref":"BatchLoader.Cache.html#clean/1","title":"BatchLoader.Cache.clean/1","type":"function","doc":""},{"ref":"BatchLoader.Cache.html#new/1","title":"BatchLoader.Cache.new/1","type":"function","doc":""},{"ref":"BatchLoader.Cache.html#value/2","title":"BatchLoader.Cache.value/2","type":"function","doc":""},{"ref":"BatchLoader.CacheStore.html","title":"BatchLoader.CacheStore","type":"module","doc":"A module which operates on the cache store (map)."},{"ref":"BatchLoader.CacheStore.html#batched?/1","title":"BatchLoader.CacheStore.batched?/1","type":"function","doc":""},{"ref":"BatchLoader.CacheStore.html#clean/1","title":"BatchLoader.CacheStore.clean/1","type":"function","doc":""},{"ref":"BatchLoader.CacheStore.html#fetch_cache/2","title":"BatchLoader.CacheStore.fetch_cache/2","type":"function","doc":""},{"ref":"BatchLoader.CacheStore.html#replace_caches/2","title":"BatchLoader.CacheStore.replace_caches/2","type":"function","doc":""},{"ref":"BatchLoader.CacheStore.html#upbatched_caches/1","title":"BatchLoader.CacheStore.upbatched_caches/1","type":"function","doc":""},{"ref":"BatchLoader.CacheStore.html#upsert_cache/2","title":"BatchLoader.CacheStore.upsert_cache/2","type":"function","doc":""},{"ref":"readme.html","title":"Overview","type":"extras","doc":"BatchLoader This package provides a generic lazy batching mechanism to avoid N+1 DB queries, HTTP queries, etc."},{"ref":"readme.html#contents","title":"Overview - Contents","type":"extras","doc":"Highlights Usage With Absinthe (GraphQL) Installation Testing"},{"ref":"readme.html#highlights","title":"Overview - Highlights","type":"extras","doc":"Generic utility to avoid N+1 DB queries, HTTP requests, etc. Adapted Elixir implementation of battle-tested tools like Haskell Haxl, JS DataLoader, Ruby BatchLoader, etc. Allows inlining the code without forcing to define extra named functions (unlike Absinthe Batch). Allows using batching with any data sources, not just Ecto (unlike Absinthe DataLoader)."},{"ref":"readme.html#usage","title":"Overview - Usage","type":"extras","doc":"With Absinthe (GraphQL) Let&#39;s imagine we have a Post GraphQL type: defmodule MyApp.PostType do use Absinthe.Schema.Notation object :post_type do field :title, :string field :user, :user_type do resolve(fn post, _, _ -&gt; user = post |&gt; Ecto.assoc(:user) |&gt; Repo.one() # N+1 DB requests {:ok, user} end) end end end This produces N+1 DB requests if we send this GraphQL request: query { posts { title user { # N+1 request per each post name } } } We can get rid of the N+1 requests by loading all Users for all Posts at once in. All we have to do is to use BatchLoader.Absinthe in the resolve function: field :user, :user_type do resolve(fn post, _, _ -&gt; BatchLoader.Absinthe.for(post.user_id, fn user_ids -&gt; Repo.all(from u in User, where: u.id in ^user_ids) # load all users at once (DB, HTTP, etc.) |&gt; Enum.map(fn user -&gt; {user.id, {:ok, user}} end) # return &quot;{user.id, result}&quot; tuples (user.id == post.user_id) end) end) end Alternatively, if you&#39;d like to use a separate named function: field :user, :user_type do resolve(fn post, _, _ -&gt; BatchLoader.Absinthe.for(post.user_id, &amp;resolved_users_by_user_ids/1) end) end def resolved_users_by_user_ids(user_ids) do Repo.all(from u in User, where: u.id in ^user_ids) |&gt; Enum.map(fn user -&gt; {user.id, {:ok, user}} end) end Finally, add BatchLoader.Absinthe.Plugin plugin to the Absinthe schema. This will allow to lazily collect information about all users which need to be loaded and then load them all at once: defmodule MyApp.Schema do use Absinthe.Schema import_types MyApp.PostType def plugins do [BatchLoader.Absinthe.Plugin] ++ Absinthe.Plugin.defaults() end end"},{"ref":"readme.html#installation","title":"Overview - Installation","type":"extras","doc":"Add batch_loader to your list of dependencies in mix.exs: def deps do [ {:batch_loader, &quot;~&gt; 0.1.0-beta.1&quot;} ] end"},{"ref":"readme.html#testing","title":"Overview - Testing","type":"extras","doc":"make install make test"}]